import tpm2_pytss
from tpm2_pytss._libtpm2_pytss import ffi as tpm2_pytss_ffi

from ...spec.structures import structures_types
from ...spec.structures.algorithm_parameters_and_structures import (
    TPM2B_DERIVE,
    TPM2B_ECC_PARAMETER,
    TPM2B_ECC_POINT,
    TPM2B_ENCRYPTED_SECRET,
    TPM2B_LABEL,
    TPM2B_PRIVATE_KEY_RSA,
    TPM2B_PUBLIC_KEY_RSA,
    TPM2B_SENSITIVE_CREATE,
    TPM2B_SENSITIVE_DATA,
    TPM2B_SYM_KEY,
    TPMI_AES_KEY_BITS,
    TPMI_ALG_ASYM_SCHEME,
    TPMI_ALG_ECC_SCHEME,
    TPMI_ALG_KEYEDHASH_SCHEME,
    TPMI_ALG_RSA_DECRYPT,
    TPMI_ALG_RSA_SCHEME,
    TPMI_CAMELLIA_KEY_BITS,
    TPMI_ECC_CURVE,
    TPMI_RSA_KEY_BITS,
    TPMI_SM4_KEY_BITS,
    TPMI_TDES_KEY_BITS,
    TPMS_ALGORITHM_DETAIL_ECC,
    TPMS_DERIVE,
    TPMS_ECC_POINT,
    TPMS_ENC_SCHEME_OAEP,
    TPMS_ENC_SCHEME_RSAES,
    TPMS_KEY_SCHEME_ECDH,
    TPMS_KEY_SCHEME_ECMQV,
    TPMS_SCHEME_ECDAA,
    TPMS_SCHEME_HASH,
    TPMS_SCHEME_HMAC,
    TPMS_SCHEME_KDF1_SP800_56A,
    TPMS_SCHEME_KDF1_SP800_108,
    TPMS_SCHEME_KDF2,
    TPMS_SCHEME_MGF1,
    TPMS_SCHEME_XOR,
    TPMS_SENSITIVE_CREATE,
    TPMS_SIG_SCHEME_ECDAA,
    TPMS_SIG_SCHEME_ECDSA,
    TPMS_SIG_SCHEME_ECSCHNORR,
    TPMS_SIG_SCHEME_RSAPSS,
    TPMS_SIG_SCHEME_RSASSA,
    TPMS_SIG_SCHEME_SM2,
    TPMS_SIGNATURE_ECC,
    TPMS_SIGNATURE_ECDAA,
    TPMS_SIGNATURE_ECDSA,
    TPMS_SIGNATURE_ECSCHNORR,
    TPMS_SIGNATURE_RSA,
    TPMS_SIGNATURE_RSAPSS,
    TPMS_SIGNATURE_RSASSA,
    TPMS_SIGNATURE_SM2,
    TPMS_SYMCIPHER_PARMS,
    TPMT_ASYM_SCHEME,
    TPMT_ECC_SCHEME,
    TPMT_KDF_SCHEME,
    TPMT_KEYEDHASH_SCHEME,
    TPMT_RSA_DECRYPT,
    TPMT_RSA_SCHEME,
    TPMT_SIG_SCHEME,
    TPMT_SIGNATURE,
    TPMT_SYM_DEF,
    TPMT_SYM_DEF_OBJECT,
    TPMU_ASYM_SCHEME,
    TPMU_ENCRYPTED_SECRET,
    TPMU_KDF_SCHEME,
    TPMU_SCHEME_KEYEDHASH,
    TPMU_SENSITIVE_CREATE,
    TPMU_SIG_SCHEME,
    TPMU_SIGNATURE,
    TPMU_SYM_DETAILS,
    TPMU_SYM_KEY_BITS,
    TPMU_SYM_MODE,
)
from ...spec.structures.attached_component_structures import (
    TPM_AE,
    TPM_AT,
    TPML_AC_CAPABILITIES,
    TPMS_AC_OUTPUT,
)
from ...spec.structures.attribute_structures import (
    TPMA_ACT,
    TPMA_ALGORITHM,
    TPMA_CC,
    TPMA_LOCALITY,
    TPMA_MEMORY,
    TPMA_MODES,
    TPMA_OBJECT,
    TPMA_PERMANENT,
    TPMA_SESSION,
    TPMA_STARTUP_CLEAR,
    TPMA_X509_KEY_USAGE,
)
from ...spec.structures.base_types import (
    BOOL,
    BYTE,
    INT8,
    INT16,
    INT32,
    INT64,
    TPM_ALGORITHM_ID,
    TPM_AUTHORIZATION_SIZE,
    TPM_KEY_BITS,
    TPM_KEY_SIZE,
    TPM_MODIFIER_INDICATOR,
    TPM_PARAMETER_SIZE,
    UINT8,
    UINT16,
    UINT32,
    UINT64,
)
from ...spec.structures.constants import (
    TPM_ALG,
    TPM_ALG_ID,
    TPM_CAP,
    TPM_CC,
    TPM_CLOCK,
    TPM_CLOCK_ADJUST,
    TPM_ECC_CURVE,
    TPM_EO,
    TPM_GENERATED,
    TPM_PS,
    TPM_PT,
    TPM_PT_PCR,
    TPM_RC,
    TPM_SE,
    TPM_SPEC,
    TPM_ST,
    TPM_SU,
)
from ...spec.structures.context_data import (
    TPM2B_CONTEXT_DATA,
    TPM2B_CONTEXT_SENSITIVE,
    TPMS_CONTEXT,
    TPMS_CONTEXT_DATA,
)
from ...spec.structures.creation_data import TPM2B_CREATION_DATA, TPMS_CREATION_DATA
from ...spec.structures.handles import TPM_HANDLE, TPM_HR, TPM_HT, TPM_RH, TPM_RS
from ...spec.structures.interface_types import (
    TPMI_ALG_ASYM,
    TPMI_ALG_CIPHER_MODE,
    TPMI_ALG_HASH,
    TPMI_ALG_KDF,
    TPMI_ALG_MAC_SCHEME,
    TPMI_ALG_SIG_SCHEME,
    TPMI_ALG_SYM,
    TPMI_ALG_SYM_MODE,
    TPMI_ALG_SYM_OBJECT,
    TPMI_DH_CONTEXT,
    TPMI_DH_ENTITY,
    TPMI_DH_OBJECT,
    TPMI_DH_PARENT,
    TPMI_DH_PCR,
    TPMI_DH_PERSISTENT,
    TPMI_DH_SAVED,
    TPMI_ECC_KEY_EXCHANGE,
    TPMI_RH_AC,
    TPMI_RH_ACT,
    TPMI_RH_CLEAR,
    TPMI_RH_ENABLES,
    TPMI_RH_ENDORSEMENT,
    TPMI_RH_HIERARCHY,
    TPMI_RH_HIERARCHY_AUTH,
    TPMI_RH_HIERARCHY_POLICY,
    TPMI_RH_LOCKOUT,
    TPMI_RH_NV_AUTH,
    TPMI_RH_NV_INDEX,
    TPMI_RH_OWNER,
    TPMI_RH_PLATFORM,
    TPMI_RH_PROVISION,
    TPMI_SH_AUTH_SESSION,
    TPMI_SH_HMAC,
    TPMI_SH_POLICY,
    TPMI_ST_COMMAND_TAG,
    TPMI_YES_NO,
)
from ...spec.structures.key_object_complex import (
    TPM2B_ID_OBJECT,
    TPM2B_PRIVATE,
    TPM2B_PRIVATE_VENDOR_SPECIFIC,
    TPM2B_PUBLIC,
    TPM2B_SENSITIVE,
    TPM2B_TEMPLATE,
    TPMI_ALG_PUBLIC,
    TPMS_ASYM_PARMS,
    TPMS_ECC_PARMS,
    TPMS_ID_OBJECT,
    TPMS_KEYEDHASH_PARMS,
    TPMS_RSA_PARMS,
    TPMT_PUBLIC,
    TPMT_PUBLIC_PARMS,
    TPMT_SENSITIVE,
    TPMU_PUBLIC_ID,
    TPMU_PUBLIC_PARMS,
    TPMU_SENSITIVE_COMPOSITE,
)
from ...spec.structures.nv_storage_structures import (
    TPM2B_NV_PUBLIC,
    TPM_NT,
    TPMA_NV,
    TPMS_NV_PIN_COUNTER_PARAMETERS,
    TPMS_NV_PUBLIC,
)
from ...spec.structures.structures import (
    TPM2B_ATTEST,
    TPM2B_AUTH,
    TPM2B_DATA,
    TPM2B_DIGEST,
    TPM2B_EVENT,
    TPM2B_IV,
    TPM2B_MAX_BUFFER,
    TPM2B_MAX_NV_BUFFER,
    TPM2B_NAME,
    TPM2B_NONCE,
    TPM2B_OPERAND,
    TPM2B_TIMEOUT,
    TPMI_ST_ATTEST,
    TPML_ACT_DATA,
    TPML_ALG,
    TPML_ALG_PROPERTY,
    TPML_CC,
    TPML_CCA,
    TPML_DIGEST,
    TPML_DIGEST_VALUES,
    TPML_ECC_CURVE,
    TPML_HANDLE,
    TPML_PCR_SELECTION,
    TPML_TAGGED_PCR_PROPERTY,
    TPML_TAGGED_POLICY,
    TPML_TAGGED_TPM_PROPERTY,
    TPMS_ACT_DATA,
    TPMS_ALG_PROPERTY,
    TPMS_ALGORITHM_DESCRIPTION,
    TPMS_ATTEST,
    TPMS_AUTH_COMMAND,
    TPMS_AUTH_RESPONSE,
    TPMS_CAPABILITY_DATA,
    TPMS_CERTIFY_INFO,
    TPMS_CLOCK_INFO,
    TPMS_COMMAND_AUDIT_INFO,
    TPMS_CREATION_INFO,
    TPMS_EMPTY,
    TPMS_NV_CERTIFY_INFO,
    TPMS_NV_DIGEST_CERTIFY_INFO,
    TPMS_PCR_SELECT,
    TPMS_PCR_SELECTION,
    TPMS_QUOTE_INFO,
    TPMS_SESSION_AUDIT_INFO,
    TPMS_TAGGED_PCR_SELECT,
    TPMS_TAGGED_POLICY,
    TPMS_TAGGED_PROPERTY,
    TPMS_TIME_ATTEST_INFO,
    TPMS_TIME_INFO,
    TPMT_HA,
    TPMT_TK_AUTH,
    TPMT_TK_CREATION,
    TPMT_TK_HASHCHECK,
    TPMT_TK_VERIFIED,
    TPMU_ATTEST,
    TPMU_CAPABILITIES,
    TPMU_HA,
    TPMU_NAME,
)

mapping = {
    BOOL: tpm2_pytss_ffi.typeof("BOOL"),
    BYTE: tpm2_pytss_ffi.typeof("BYTE"),
    INT16: tpm2_pytss_ffi.typeof("INT16"),
    INT32: tpm2_pytss_ffi.typeof("INT32"),
    INT64: tpm2_pytss_ffi.typeof("INT64"),
    INT8: tpm2_pytss_ffi.typeof("INT8"),
    TPM_AE: None,
    TPM_ALG: tpm2_pytss.types.TPM2_ALG,
    TPM_ALG_ID: tpm2_pytss.constants.TPM2_ALG_ID,
    TPM_ALGORITHM_ID: tpm2_pytss_ffi.typeof("TPM2_ALGORITHM_ID"),
    TPM_AT: tpm2_pytss_ffi.typeof("TPM_AT"),
    TPM_AUTHORIZATION_SIZE: tpm2_pytss_ffi.typeof("TPM2_AUTHORIZATION_SIZE"),
    TPM_CAP: tpm2_pytss.constants.TPM2_CAP,
    TPM_CC: tpm2_pytss.constants.TPM2_CC,
    TPM_CLOCK: tpm2_pytss.constants.TPM2_CLOCK,
    TPM_CLOCK_ADJUST: tpm2_pytss.constants.TPM2_CLOCK_ADJUST,
    TPM_ECC_CURVE: tpm2_pytss.types.TPM2_ECC_CURVE,
    TPM_EO: tpm2_pytss.constants.TPM2_EO,
    TPM_GENERATED: tpm2_pytss.constants.TPM2_GENERATED,
    TPM_HANDLE: tpm2_pytss.types.TPM2_HANDLE,
    TPM_HR: tpm2_pytss.types.TPM2_HR,
    TPM_HT: tpm2_pytss.constants.TPM2_HT,
    TPM_KEY_BITS: tpm2_pytss_ffi.typeof("TPM2_KEY_BITS"),
    TPM_KEY_SIZE: tpm2_pytss_ffi.typeof("TPM2_KEY_SIZE"),
    TPM_MODIFIER_INDICATOR: tpm2_pytss_ffi.typeof("TPM2_MODIFIER_INDICATOR"),
    TPM_NT: tpm2_pytss.constants.TPM2_NT,
    TPM_PARAMETER_SIZE: tpm2_pytss_ffi.typeof("TPM2_PARAMETER_SIZE"),
    TPM_PS: tpm2_pytss.constants.TPM2_PS,
    TPM_PT: tpm2_pytss.constants.TPM2_PT,
    TPM_PT_PCR: tpm2_pytss.constants.TPM2_PT_PCR,
    TPM_RC: tpm2_pytss.constants.TPM2_RC,
    TPM_RH: tpm2_pytss.constants.TPM2_RH,
    TPM_RS: None,
    TPM_SE: tpm2_pytss.constants.TPM2_SE,
    TPM_SPEC: tpm2_pytss.constants.TPM2_SPEC,
    TPM_ST: tpm2_pytss.constants.TPM2_ST,
    TPM_SU: tpm2_pytss.constants.TPM2_SU,
    TPM2B_ATTEST: tpm2_pytss.types.TPM2B_ATTEST,
    TPM2B_AUTH: tpm2_pytss.types.TPM2B_AUTH,
    TPM2B_CONTEXT_DATA: tpm2_pytss.types.TPM2B_CONTEXT_DATA,
    TPM2B_CONTEXT_SENSITIVE: tpm2_pytss.types.TPM2B_CONTEXT_SENSITIVE,
    TPM2B_CREATION_DATA: tpm2_pytss.types.TPM2B_CREATION_DATA,
    TPM2B_DATA: tpm2_pytss.types.TPM2B_DATA,
    TPM2B_DERIVE: None,
    TPM2B_DIGEST: tpm2_pytss.types.TPM2B_DIGEST,
    TPM2B_ECC_PARAMETER: tpm2_pytss.types.TPM2B_ECC_PARAMETER,
    TPM2B_ECC_POINT: tpm2_pytss.types.TPM2B_ECC_POINT,
    TPM2B_ENCRYPTED_SECRET: tpm2_pytss.types.TPM2B_ENCRYPTED_SECRET,
    TPM2B_EVENT: tpm2_pytss.types.TPM2B_EVENT,
    TPM2B_ID_OBJECT: tpm2_pytss.types.TPM2B_ID_OBJECT,
    TPM2B_IV: tpm2_pytss.types.TPM2B_IV,
    TPM2B_LABEL: None,
    TPM2B_MAX_BUFFER: tpm2_pytss.types.TPM2B_MAX_BUFFER,
    TPM2B_MAX_NV_BUFFER: tpm2_pytss.types.TPM2B_MAX_NV_BUFFER,
    TPM2B_NAME: tpm2_pytss.types.TPM2B_NAME,
    TPM2B_NONCE: tpm2_pytss.types.TPM2B_NONCE,
    TPM2B_NV_PUBLIC: tpm2_pytss.types.TPM2B_NV_PUBLIC,
    TPM2B_OPERAND: tpm2_pytss.types.TPM2B_OPERAND,
    TPM2B_PRIVATE: tpm2_pytss.types.TPM2B_PRIVATE,
    TPM2B_PRIVATE_KEY_RSA: tpm2_pytss.types.TPM2B_PRIVATE_KEY_RSA,
    TPM2B_PRIVATE_VENDOR_SPECIFIC: tpm2_pytss.types.TPM2B_PRIVATE_VENDOR_SPECIFIC,
    TPM2B_PUBLIC: tpm2_pytss.types.TPM2B_PUBLIC,
    TPM2B_PUBLIC_KEY_RSA: tpm2_pytss.types.TPM2B_PUBLIC_KEY_RSA,
    TPM2B_SENSITIVE: tpm2_pytss.types.TPM2B_SENSITIVE,
    TPM2B_SENSITIVE_CREATE: tpm2_pytss.types.TPM2B_SENSITIVE_CREATE,
    TPM2B_SENSITIVE_DATA: tpm2_pytss.types.TPM2B_SENSITIVE_DATA,
    TPM2B_SYM_KEY: tpm2_pytss.types.TPM2B_SYM_KEY,
    TPM2B_TEMPLATE: tpm2_pytss.types.TPM2B_TEMPLATE,
    TPM2B_TIMEOUT: tpm2_pytss.types.TPM2B_TIMEOUT,
    TPMA_ACT: tpm2_pytss_ffi.typeof("TPMA_ACT"),
    TPMA_ALGORITHM: tpm2_pytss.constants.TPMA_ALGORITHM,
    TPMA_CC: tpm2_pytss.constants.TPMA_CC,
    TPMA_LOCALITY: tpm2_pytss.constants.TPMA_LOCALITY,
    TPMA_MEMORY: tpm2_pytss.constants.TPMA_MEMORY,
    TPMA_MODES: tpm2_pytss_ffi.typeof("TPMA_MODES"),
    TPMA_NV: tpm2_pytss.constants.TPMA_NV,
    TPMA_OBJECT: tpm2_pytss.constants.TPMA_OBJECT,
    TPMA_PERMANENT: tpm2_pytss.constants.TPMA_PERMANENT,
    TPMA_SESSION: tpm2_pytss.constants.TPMA_SESSION,
    TPMA_STARTUP_CLEAR: tpm2_pytss.constants.TPMA_STARTUP,
    TPMA_X509_KEY_USAGE: tpm2_pytss_ffi.typeof("TPMA_X509_KEY_USAGE"),
    TPMI_AES_KEY_BITS: tpm2_pytss_ffi.typeof("TPMI_AES_KEY_BITS"),
    TPMI_ALG_ASYM: tpm2_pytss_ffi.typeof("TPMI_ALG_ASYM"),
    TPMI_ALG_ASYM_SCHEME: tpm2_pytss_ffi.typeof("TPMI_ALG_ASYM_SCHEME"),
    TPMI_ALG_CIPHER_MODE: tpm2_pytss_ffi.typeof("TPMI_ALG_CIPHER_MODE"),
    TPMI_ALG_ECC_SCHEME: tpm2_pytss_ffi.typeof("TPMI_ALG_ECC_SCHEME"),
    TPMI_ALG_HASH: tpm2_pytss_ffi.typeof("TPMI_ALG_HASH"),
    TPMI_ALG_KDF: tpm2_pytss_ffi.typeof("TPMI_ALG_KDF"),
    TPMI_ALG_KEYEDHASH_SCHEME: tpm2_pytss_ffi.typeof("TPMI_ALG_KEYEDHASH_SCHEME"),
    TPMI_ALG_MAC_SCHEME: tpm2_pytss_ffi.typeof("TPMI_ALG_MAC_SCHEME"),
    TPMI_ALG_PUBLIC: tpm2_pytss_ffi.typeof("TPMI_ALG_PUBLIC"),
    TPMI_ALG_RSA_DECRYPT: tpm2_pytss_ffi.typeof("TPMI_ALG_RSA_DECRYPT"),
    TPMI_ALG_RSA_SCHEME: tpm2_pytss_ffi.typeof("TPMI_ALG_RSA_SCHEME"),
    TPMI_ALG_SIG_SCHEME: tpm2_pytss_ffi.typeof("TPMI_ALG_SIG_SCHEME"),
    TPMI_ALG_SYM: tpm2_pytss_ffi.typeof("TPMI_ALG_SYM"),
    TPMI_ALG_SYM_MODE: tpm2_pytss_ffi.typeof("TPMI_ALG_SYM_MODE"),
    TPMI_ALG_SYM_OBJECT: tpm2_pytss_ffi.typeof("TPMI_ALG_SYM_OBJECT"),
    TPMI_CAMELLIA_KEY_BITS: tpm2_pytss_ffi.typeof("TPMI_CAMELLIA_KEY_BITS"),
    TPMI_DH_CONTEXT: tpm2_pytss_ffi.typeof("TPMI_DH_CONTEXT"),
    TPMI_DH_ENTITY: tpm2_pytss_ffi.typeof("TPMI_DH_ENTITY"),
    TPMI_DH_OBJECT: tpm2_pytss_ffi.typeof("TPMI_DH_OBJECT"),
    TPMI_DH_PARENT: None,
    TPMI_DH_PCR: tpm2_pytss_ffi.typeof("TPMI_DH_PCR"),
    TPMI_DH_PERSISTENT: tpm2_pytss_ffi.typeof("TPMI_DH_PERSISTENT"),
    TPMI_DH_SAVED: tpm2_pytss_ffi.typeof("TPMI_DH_SAVED"),
    TPMI_ECC_CURVE: tpm2_pytss_ffi.typeof("TPMI_ECC_CURVE"),
    TPMI_ECC_KEY_EXCHANGE: tpm2_pytss_ffi.typeof("TPMI_ECC_KEY_EXCHANGE"),
    TPMI_RH_AC: tpm2_pytss_ffi.typeof("TPMI_RH_AC"),
    TPMI_RH_ACT: tpm2_pytss_ffi.typeof("TPMI_RH_ACT"),
    TPMI_RH_CLEAR: tpm2_pytss_ffi.typeof("TPMI_RH_CLEAR"),
    TPMI_RH_ENABLES: tpm2_pytss_ffi.typeof("TPMI_RH_ENABLES"),
    TPMI_RH_ENDORSEMENT: tpm2_pytss_ffi.typeof("TPMI_RH_ENDORSEMENT"),
    TPMI_RH_HIERARCHY: tpm2_pytss_ffi.typeof("TPMI_RH_HIERARCHY"),
    TPMI_RH_HIERARCHY_AUTH: tpm2_pytss_ffi.typeof("TPMI_RH_HIERARCHY_AUTH"),
    TPMI_RH_HIERARCHY_POLICY: tpm2_pytss_ffi.typeof("TPMI_RH_HIERARCHY_POLICY"),
    TPMI_RH_LOCKOUT: tpm2_pytss_ffi.typeof("TPMI_RH_LOCKOUT"),
    TPMI_RH_NV_AUTH: tpm2_pytss_ffi.typeof("TPMI_RH_NV_AUTH"),
    TPMI_RH_NV_INDEX: tpm2_pytss_ffi.typeof("TPMI_RH_NV_INDEX"),
    TPMI_RH_OWNER: tpm2_pytss_ffi.typeof("TPMI_RH_OWNER"),
    TPMI_RH_PLATFORM: tpm2_pytss_ffi.typeof("TPMI_RH_PLATFORM"),
    TPMI_RH_PROVISION: tpm2_pytss_ffi.typeof("TPMI_RH_PROVISION"),
    TPMI_RSA_KEY_BITS: tpm2_pytss_ffi.typeof("TPMI_RSA_KEY_BITS"),
    TPMI_SH_AUTH_SESSION: tpm2_pytss_ffi.typeof("TPMI_SH_AUTH_SESSION"),
    TPMI_SH_HMAC: tpm2_pytss_ffi.typeof("TPMI_SH_HMAC"),
    TPMI_SH_POLICY: tpm2_pytss_ffi.typeof("TPMI_SH_POLICY"),
    TPMI_SM4_KEY_BITS: tpm2_pytss_ffi.typeof("TPMI_SM4_KEY_BITS"),
    TPMI_ST_ATTEST: tpm2_pytss_ffi.typeof("TPMI_ST_ATTEST"),
    TPMI_ST_COMMAND_TAG: tpm2_pytss_ffi.typeof("TPMI_ST_COMMAND_TAG"),
    TPMI_TDES_KEY_BITS: None,
    TPMI_YES_NO: tpm2_pytss_ffi.typeof("TPMI_YES_NO"),
    TPML_AC_CAPABILITIES: tpm2_pytss.types.TPML_AC_CAPABILITIES,
    TPML_ACT_DATA: tpm2_pytss_ffi.typeof("TPML_ACT_DATA"),
    TPML_ALG: tpm2_pytss.types.TPML_ALG,
    TPML_ALG_PROPERTY: tpm2_pytss.types.TPML_ALG_PROPERTY,
    TPML_CC: tpm2_pytss.types.TPML_CC,
    TPML_CCA: tpm2_pytss.types.TPML_CCA,
    TPML_DIGEST: tpm2_pytss.types.TPML_DIGEST,
    TPML_DIGEST_VALUES: tpm2_pytss.types.TPML_DIGEST_VALUES,
    TPML_ECC_CURVE: tpm2_pytss.types.TPML_ECC_CURVE,
    TPML_HANDLE: tpm2_pytss.types.TPML_HANDLE,
    TPML_PCR_SELECTION: tpm2_pytss.types.TPML_PCR_SELECTION,
    TPML_TAGGED_PCR_PROPERTY: tpm2_pytss.types.TPML_TAGGED_PCR_PROPERTY,
    TPML_TAGGED_POLICY: tpm2_pytss_ffi.typeof("TPML_TAGGED_POLICY"),
    TPML_TAGGED_TPM_PROPERTY: tpm2_pytss.types.TPML_TAGGED_TPM_PROPERTY,
    TPMS_AC_OUTPUT: tpm2_pytss.types.TPMS_AC_OUTPUT,
    TPMS_ACT_DATA: tpm2_pytss_ffi.typeof("TPMS_ACT_DATA"),
    TPMS_ALG_PROPERTY: tpm2_pytss.types.TPMS_ALG_PROPERTY,
    TPMS_ALGORITHM_DESCRIPTION: tpm2_pytss.types.TPMS_ALGORITHM_DESCRIPTION,
    TPMS_ALGORITHM_DETAIL_ECC: tpm2_pytss.types.TPMS_ALGORITHM_DETAIL_ECC,
    TPMS_ASYM_PARMS: tpm2_pytss.types.TPMS_ASYM_PARMS,
    TPMS_ATTEST: tpm2_pytss.types.TPMS_ATTEST,
    TPMS_AUTH_COMMAND: tpm2_pytss.types.TPMS_AUTH_COMMAND,
    TPMS_AUTH_RESPONSE: tpm2_pytss.types.TPMS_AUTH_RESPONSE,
    TPMS_CAPABILITY_DATA: tpm2_pytss.types.TPMS_CAPABILITY_DATA,
    TPMS_CERTIFY_INFO: tpm2_pytss.types.TPMS_CERTIFY_INFO,
    TPMS_CLOCK_INFO: tpm2_pytss.types.TPMS_CLOCK_INFO,
    TPMS_COMMAND_AUDIT_INFO: tpm2_pytss.types.TPMS_COMMAND_AUDIT_INFO,
    TPMS_CONTEXT: tpm2_pytss.types.TPMS_CONTEXT,
    TPMS_CONTEXT_DATA: tpm2_pytss.types.TPMS_CONTEXT_DATA,
    TPMS_CREATION_DATA: tpm2_pytss.types.TPMS_CREATION_DATA,
    TPMS_CREATION_INFO: tpm2_pytss.types.TPMS_CREATION_INFO,
    TPMS_DERIVE: None,
    TPMS_ECC_PARMS: tpm2_pytss.types.TPMS_ECC_PARMS,
    TPMS_ECC_POINT: tpm2_pytss.types.TPMS_ECC_POINT,
    TPMS_EMPTY: tpm2_pytss.types.TPMS_EMPTY,
    TPMS_ENC_SCHEME_OAEP: tpm2_pytss_ffi.typeof("TPMS_ENC_SCHEME_OAEP"),
    TPMS_ENC_SCHEME_RSAES: tpm2_pytss_ffi.typeof("TPMS_ENC_SCHEME_RSAES"),
    TPMS_ID_OBJECT: tpm2_pytss.types.TPMS_ID_OBJECT,
    TPMS_KEY_SCHEME_ECDH: tpm2_pytss_ffi.typeof("TPMS_KEY_SCHEME_ECDH"),
    TPMS_KEY_SCHEME_ECMQV: tpm2_pytss_ffi.typeof("TPMS_KEY_SCHEME_ECMQV"),
    TPMS_KEYEDHASH_PARMS: tpm2_pytss.types.TPMS_KEYEDHASH_PARMS,
    TPMS_NV_CERTIFY_INFO: tpm2_pytss.types.TPMS_NV_CERTIFY_INFO,
    TPMS_NV_DIGEST_CERTIFY_INFO: tpm2_pytss_ffi.typeof("TPMS_NV_DIGEST_CERTIFY_INFO"),
    TPMS_NV_PIN_COUNTER_PARAMETERS: tpm2_pytss.types.TPMS_NV_PIN_COUNTER_PARAMETERS,
    TPMS_NV_PUBLIC: tpm2_pytss.types.TPMS_NV_PUBLIC,
    TPMS_PCR_SELECT: tpm2_pytss.types.TPMS_PCR_SELECT,
    TPMS_PCR_SELECTION: tpm2_pytss.types.TPMS_PCR_SELECTION,
    TPMS_QUOTE_INFO: tpm2_pytss.types.TPMS_QUOTE_INFO,
    TPMS_RSA_PARMS: tpm2_pytss.types.TPMS_RSA_PARMS,
    TPMS_SCHEME_ECDAA: tpm2_pytss.types.TPMS_SCHEME_ECDAA,
    TPMS_SCHEME_HASH: tpm2_pytss.types.TPMS_SCHEME_HASH,
    TPMS_SCHEME_HMAC: tpm2_pytss_ffi.typeof("TPMS_SCHEME_HMAC"),
    TPMS_SCHEME_MGF1: tpm2_pytss_ffi.typeof("TPMS_SCHEME_MGF1"),
    TPMS_SCHEME_KDF1_SP800_108: tpm2_pytss_ffi.typeof("TPMS_SCHEME_KDF1_SP800_108"),
    TPMS_SCHEME_KDF1_SP800_56A: tpm2_pytss_ffi.typeof("TPMS_SCHEME_KDF1_SP800_56A"),
    TPMS_SCHEME_KDF2: tpm2_pytss_ffi.typeof("TPMS_SCHEME_KDF2"),
    TPMS_SCHEME_XOR: tpm2_pytss.types.TPMS_SCHEME_XOR,
    TPMS_SENSITIVE_CREATE: tpm2_pytss.types.TPMS_SENSITIVE_CREATE,
    TPMS_SESSION_AUDIT_INFO: tpm2_pytss.types.TPMS_SESSION_AUDIT_INFO,
    TPMS_SIG_SCHEME_ECDAA: tpm2_pytss_ffi.typeof("TPMS_SIG_SCHEME_ECDAA"),
    TPMS_SIG_SCHEME_ECDSA: tpm2_pytss_ffi.typeof("TPMS_SIG_SCHEME_ECDSA"),
    TPMS_SIG_SCHEME_ECSCHNORR: tpm2_pytss_ffi.typeof("TPMS_SIG_SCHEME_ECSCHNORR"),
    TPMS_SIG_SCHEME_RSAPSS: tpm2_pytss_ffi.typeof("TPMS_SIG_SCHEME_RSAPSS"),
    TPMS_SIG_SCHEME_RSASSA: tpm2_pytss_ffi.typeof("TPMS_SIG_SCHEME_RSASSA"),
    TPMS_SIG_SCHEME_SM2: tpm2_pytss_ffi.typeof("TPMS_SIG_SCHEME_SM2"),
    TPMS_SIGNATURE_ECC: tpm2_pytss.types.TPMS_SIGNATURE_ECC,
    TPMS_SIGNATURE_ECDAA: tpm2_pytss_ffi.typeof("TPMS_SIGNATURE_ECDAA"),
    TPMS_SIGNATURE_ECDSA: tpm2_pytss_ffi.typeof("TPMS_SIGNATURE_ECDSA"),
    TPMS_SIGNATURE_ECSCHNORR: tpm2_pytss_ffi.typeof("TPMS_SIGNATURE_ECSCHNORR"),
    TPMS_SIGNATURE_RSA: tpm2_pytss_ffi.typeof("TPMS_SIGNATURE_RSA"),
    TPMS_SIGNATURE_RSAPSS: tpm2_pytss_ffi.typeof("TPMS_SIGNATURE_RSAPSS"),
    TPMS_SIGNATURE_RSASSA: tpm2_pytss_ffi.typeof("TPMS_SIGNATURE_RSASSA"),
    TPMS_SIGNATURE_SM2: tpm2_pytss_ffi.typeof("TPMS_SIGNATURE_SM2"),
    TPMS_SYMCIPHER_PARMS: tpm2_pytss.types.TPMS_SYMCIPHER_PARMS,
    TPMS_TAGGED_PCR_SELECT: tpm2_pytss.types.TPMS_TAGGED_PCR_SELECT,
    TPMS_TAGGED_POLICY: tpm2_pytss_ffi.typeof("TPMS_TAGGED_POLICY"),
    TPMS_TAGGED_PROPERTY: tpm2_pytss.types.TPMS_TAGGED_PROPERTY,
    TPMS_TIME_ATTEST_INFO: tpm2_pytss.types.TPMS_TIME_ATTEST_INFO,
    TPMS_TIME_INFO: tpm2_pytss.types.TPMS_TIME_INFO,
    TPMT_ASYM_SCHEME: tpm2_pytss.types.TPMT_ASYM_SCHEME,
    TPMT_ECC_SCHEME: tpm2_pytss.types.TPMT_ECC_SCHEME,
    TPMT_HA: tpm2_pytss.types.TPMT_HA,
    TPMT_KDF_SCHEME: tpm2_pytss.types.TPMT_KDF_SCHEME,
    TPMT_KEYEDHASH_SCHEME: tpm2_pytss.types.TPMT_KEYEDHASH_SCHEME,
    TPMT_PUBLIC: tpm2_pytss.types.TPMT_PUBLIC,
    TPMT_PUBLIC_PARMS: tpm2_pytss.types.TPMT_PUBLIC_PARMS,
    TPMT_RSA_DECRYPT: tpm2_pytss.types.TPMT_RSA_DECRYPT,
    TPMT_RSA_SCHEME: tpm2_pytss.types.TPMT_RSA_SCHEME,
    TPMT_SENSITIVE: tpm2_pytss.types.TPMT_SENSITIVE,
    TPMT_SIG_SCHEME: tpm2_pytss.types.TPMT_SIG_SCHEME,
    TPMT_SIGNATURE: tpm2_pytss.types.TPMT_SIGNATURE,
    TPMT_SYM_DEF: tpm2_pytss.types.TPMT_SYM_DEF,
    TPMT_SYM_DEF_OBJECT: tpm2_pytss.types.TPMT_SYM_DEF_OBJECT,
    TPMT_TK_AUTH: tpm2_pytss.types.TPMT_TK_AUTH,
    TPMT_TK_CREATION: tpm2_pytss.types.TPMT_TK_CREATION,
    TPMT_TK_HASHCHECK: tpm2_pytss.types.TPMT_TK_HASHCHECK,
    TPMT_TK_VERIFIED: tpm2_pytss.types.TPMT_TK_VERIFIED,
    TPMU_ASYM_SCHEME: tpm2_pytss.types.TPMU_ASYM_SCHEME,
    TPMU_ATTEST: tpm2_pytss.types.TPMU_ATTEST,
    TPMU_CAPABILITIES: tpm2_pytss.types.TPMU_CAPABILITIES,
    TPMU_ENCRYPTED_SECRET: tpm2_pytss_ffi.typeof("TPMU_ENCRYPTED_SECRET"),
    TPMU_HA: tpm2_pytss.types.TPMU_HA,
    TPMU_KDF_SCHEME: tpm2_pytss.types.TPMU_KDF_SCHEME,
    TPMU_NAME: tpm2_pytss_ffi.typeof("TPMU_NAME"),
    TPMU_PUBLIC_ID: tpm2_pytss.types.TPMU_PUBLIC_ID,
    TPMU_PUBLIC_PARMS: tpm2_pytss.types.TPMU_PUBLIC_PARMS,
    TPMU_SCHEME_KEYEDHASH: tpm2_pytss.types.TPMU_SCHEME_KEYEDHASH,
    TPMU_SENSITIVE_COMPOSITE: tpm2_pytss.types.TPMU_SENSITIVE_COMPOSITE,
    TPMU_SENSITIVE_CREATE: None,
    TPMU_SIG_SCHEME: tpm2_pytss.types.TPMU_SIG_SCHEME,
    TPMU_SIGNATURE: tpm2_pytss.types.TPMU_SIGNATURE,
    TPMU_SYM_DETAILS: None,
    TPMU_SYM_KEY_BITS: tpm2_pytss.types.TPMU_SYM_KEY_BITS,
    TPMU_SYM_MODE: tpm2_pytss.types.TPMU_SYM_MODE,
    UINT16: int,
    UINT32: int,
    UINT64: int,
    UINT8: int,
}


# Import-time sanity check: verify that all relevant types from structures are in mapping
assert sorted(mapping.keys(), key=lambda e: e.__name__) == structures_types
