from ..common.values import tpm_dataclass
from ..structures.constants import TPM_CC
from ..structures.interface_types import (
    TPMI_DH_CONTEXT,
    TPMI_DH_ENTITY,
    TPMI_DH_OBJECT,
    TPMI_DH_PARENT,
    TPMI_DH_PCR,
    TPMI_RH_AC,
    TPMI_RH_ACT,
    TPMI_RH_CLEAR,
    TPMI_RH_ENDORSEMENT,
    TPMI_RH_HIERARCHY,
    TPMI_RH_HIERARCHY_AUTH,
    TPMI_RH_HIERARCHY_POLICY,
    TPMI_RH_LOCKOUT,
    TPMI_RH_NV_AUTH,
    TPMI_RH_NV_INDEX,
    TPMI_RH_PLATFORM,
    TPMI_RH_PROVISION,
    TPMI_SH_HMAC,
    TPMI_SH_POLICY,
)


@tpm_dataclass
class TPMS_COMMAND_HANDLES_NV_UNDEFINE_SPACE_SPECIAL:
    nvIndex: TPMI_RH_NV_INDEX
    platform: TPMI_RH_PLATFORM


@tpm_dataclass
class TPMS_COMMAND_HANDLES_EVICT_CONTROL:
    auth: TPMI_RH_PROVISION
    objectHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_HIERARCHY_CONTROL:
    authHandle: TPMI_RH_HIERARCHY


@tpm_dataclass
class TPMS_COMMAND_HANDLES_NV_UNDEFINE_SPACE:
    authHandle: TPMI_RH_PROVISION
    nvIndex: TPMI_RH_NV_INDEX


@tpm_dataclass
class TPMS_COMMAND_HANDLES_CHANGE_EPS:
    authHandle: TPMI_RH_PLATFORM


@tpm_dataclass
class TPMS_COMMAND_HANDLES_CHANGE_PPS:
    authHandle: TPMI_RH_PLATFORM


@tpm_dataclass
class TPMS_COMMAND_HANDLES_CLEAR:
    authHandle: TPMI_RH_CLEAR


@tpm_dataclass
class TPMS_COMMAND_HANDLES_CLEAR_CONTROL:
    auth: TPMI_RH_CLEAR


@tpm_dataclass
class TPMS_COMMAND_HANDLES_CLOCK_SET:
    auth: TPMI_RH_PROVISION


@tpm_dataclass
class TPMS_COMMAND_HANDLES_HIERARCHY_CHANGE_AUTH:
    authHandle: TPMI_RH_HIERARCHY_AUTH


@tpm_dataclass
class TPMS_COMMAND_HANDLES_NV_DEFINE_SPACE:
    authHandle: TPMI_RH_PROVISION


@tpm_dataclass
class TPMS_COMMAND_HANDLES_PCR_ALLOCATE:
    authHandle: TPMI_RH_PLATFORM


@tpm_dataclass
class TPMS_COMMAND_HANDLES_PCR_SET_AUTH_POLICY:
    authHandle: TPMI_RH_PLATFORM


@tpm_dataclass
class TPMS_COMMAND_HANDLES_PP_COMMANDS:
    auth: TPMI_RH_PLATFORM


@tpm_dataclass
class TPMS_COMMAND_HANDLES_SET_PRIMARY_POLICY:
    authHandle: TPMI_RH_HIERARCHY_POLICY


@tpm_dataclass
class TPMS_COMMAND_HANDLES_FIELD_UPGRADE_START:
    authorization: TPMI_RH_PLATFORM
    keyHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_INCREMENTAL_SELF_TEST:
    pass


@tpm_dataclass
class TPMS_COMMAND_HANDLES_SELF_TEST:
    pass


@tpm_dataclass
class TPMS_COMMAND_HANDLES_STARTUP:
    pass


@tpm_dataclass
class TPMS_COMMAND_HANDLES_SHUTDOWN:
    pass


@tpm_dataclass
class TPMS_COMMAND_HANDLES_CLOCK_RATE_ADJUST:
    auth: TPMI_RH_PROVISION


@tpm_dataclass
class TPMS_COMMAND_HANDLES_CREATE_PRIMARY:
    primaryHandle: TPMI_RH_HIERARCHY


@tpm_dataclass
class TPMS_COMMAND_HANDLES_NV_GLOBAL_WRITE_LOCK:
    authHandle: TPMI_RH_PROVISION


@tpm_dataclass
class TPMS_COMMAND_HANDLES_GET_COMMAND_AUDIT_DIGEST:
    privacyHandle: TPMI_RH_ENDORSEMENT
    signHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_NV_INCREMENT:
    authHandle: TPMI_RH_NV_AUTH
    nvIndex: TPMI_RH_NV_INDEX


@tpm_dataclass
class TPMS_COMMAND_HANDLES_NV_SET_BITS:
    authHandle: TPMI_RH_NV_AUTH
    nvIndex: TPMI_RH_NV_INDEX


@tpm_dataclass
class TPMS_COMMAND_HANDLES_NV_EXTEND:
    authHandle: TPMI_RH_NV_AUTH
    nvIndex: TPMI_RH_NV_INDEX


@tpm_dataclass
class TPMS_COMMAND_HANDLES_NV_WRITE:
    authHandle: TPMI_RH_NV_AUTH
    nvIndex: TPMI_RH_NV_INDEX


@tpm_dataclass
class TPMS_COMMAND_HANDLES_NV_WRITE_LOCK:
    authHandle: TPMI_RH_NV_AUTH
    nvIndex: TPMI_RH_NV_INDEX


@tpm_dataclass
class TPMS_COMMAND_HANDLES_DICTIONARY_ATTACK_LOCK_RESET:
    lockHandle: TPMI_RH_LOCKOUT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_DICTIONARY_ATTACK_PARAMETERS:
    lockHandle: TPMI_RH_LOCKOUT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_NV_CHANGE_AUTH:
    nvIndex: TPMI_RH_NV_INDEX


@tpm_dataclass
class TPMS_COMMAND_HANDLES_PCR_EVENT:
    pcrHandle: TPMI_DH_PCR


@tpm_dataclass
class TPMS_COMMAND_HANDLES_PCR_RESET:
    pcrHandle: TPMI_DH_PCR


@tpm_dataclass
class TPMS_COMMAND_HANDLES_SEQUENCE_COMPLETE:
    sequenceHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_SET_ALGORITHM_SET:
    authHandle: TPMI_RH_PLATFORM


@tpm_dataclass
class TPMS_COMMAND_HANDLES_SET_COMMAND_CODE_AUDIT_STATUS:
    auth: TPMI_RH_PROVISION


@tpm_dataclass
class TPMS_COMMAND_HANDLES_FIELD_UPGRADE_DATA:
    pass


@tpm_dataclass
class TPMS_COMMAND_HANDLES_STIR_RANDOM:
    pass


@tpm_dataclass
class TPMS_COMMAND_HANDLES_ACTIVATE_CREDENTIAL:
    activateHandle: TPMI_DH_OBJECT
    keyHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_CERTIFY:
    objectHandle: TPMI_DH_OBJECT
    signHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_POLICY_NV:
    authHandle: TPMI_RH_NV_AUTH
    nvIndex: TPMI_RH_NV_INDEX
    policySession: TPMI_SH_POLICY


@tpm_dataclass
class TPMS_COMMAND_HANDLES_CERTIFY_CREATION:
    signHandle: TPMI_DH_OBJECT
    objectHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_DUPLICATE:
    objectHandle: TPMI_DH_OBJECT
    newParentHandle: TPMI_DH_OBJECT.plus()


@tpm_dataclass
class TPMS_COMMAND_HANDLES_GET_TIME:
    privacyAdminHandle: TPMI_RH_ENDORSEMENT
    signHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_GET_SESSION_AUDIT_DIGEST:
    privacyAdminHandle: TPMI_RH_ENDORSEMENT
    signHandle: TPMI_DH_OBJECT
    sessionHandle: TPMI_SH_HMAC


@tpm_dataclass
class TPMS_COMMAND_HANDLES_NV_READ:
    authHandle: TPMI_RH_NV_AUTH
    nvIndex: TPMI_RH_NV_INDEX


@tpm_dataclass
class TPMS_COMMAND_HANDLES_NV_READ_LOCK:
    authHandle: TPMI_RH_NV_AUTH
    nvIndex: TPMI_RH_NV_INDEX


@tpm_dataclass
class TPMS_COMMAND_HANDLES_OBJECT_CHANGE_AUTH:
    objectHandle: TPMI_DH_OBJECT
    parentHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_POLICY_SECRET:
    authHandle: TPMI_DH_ENTITY
    policySession: TPMI_SH_POLICY


@tpm_dataclass
class TPMS_COMMAND_HANDLES_REWRAP:
    oldParent: TPMI_DH_OBJECT.plus()
    newParent: TPMI_DH_OBJECT.plus()


@tpm_dataclass
class TPMS_COMMAND_HANDLES_CREATE:
    parentHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_ECDH_Z_GEN:
    keyHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_HMAC:
    handle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_IMPORT:
    parentHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_LOAD:
    parentHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_QUOTE:
    signHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_RSA_DECRYPT:
    keyHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_HMAC_START:
    handle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_SEQUENCE_UPDATE:
    sequenceHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_SIGN:
    keyHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_UNSEAL:
    itemHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_POLICY_SIGNED:
    authObject: TPMI_DH_OBJECT
    policySession: TPMI_SH_POLICY


@tpm_dataclass
class TPMS_COMMAND_HANDLES_CONTEXT_LOAD:
    pass


@tpm_dataclass
class TPMS_COMMAND_HANDLES_CONTEXT_SAVE:
    saveHandle: TPMI_DH_CONTEXT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_ECDH_KEY_GEN:
    keyHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_ENCRYPT_DECRYPT:
    keyHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_FLUSH_CONTEXT:
    flushHandle: TPMI_DH_CONTEXT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_LOAD_EXTERNAL:
    pass


@tpm_dataclass
class TPMS_COMMAND_HANDLES_MAKE_CREDENTIAL:
    handle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_NV_READ_PUBLIC:
    nvIndex: TPMI_RH_NV_INDEX


@tpm_dataclass
class TPMS_COMMAND_HANDLES_POLICY_AUTHORIZE:
    policySession: TPMI_SH_POLICY


@tpm_dataclass
class TPMS_COMMAND_HANDLES_POLICY_AUTH_VALUE:
    policySession: TPMI_SH_POLICY


@tpm_dataclass
class TPMS_COMMAND_HANDLES_POLICY_COMMAND_CODE:
    policySession: TPMI_SH_POLICY


@tpm_dataclass
class TPMS_COMMAND_HANDLES_POLICY_COUNTER_TIMER:
    policySession: TPMI_SH_POLICY


@tpm_dataclass
class TPMS_COMMAND_HANDLES_POLICY_CP_HASH:
    policySession: TPMI_SH_POLICY


@tpm_dataclass
class TPMS_COMMAND_HANDLES_POLICY_LOCALITY:
    policySession: TPMI_SH_POLICY


@tpm_dataclass
class TPMS_COMMAND_HANDLES_POLICY_NAME_HASH:
    policySession: TPMI_SH_POLICY


@tpm_dataclass
class TPMS_COMMAND_HANDLES_POLICY_OR:
    policySession: TPMI_SH_POLICY


@tpm_dataclass
class TPMS_COMMAND_HANDLES_POLICY_TICKET:
    policySession: TPMI_SH_POLICY


@tpm_dataclass
class TPMS_COMMAND_HANDLES_READ_PUBLIC:
    objectHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_RSA_ENCRYPT:
    keyHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_START_AUTH_SESSION:
    tpmKey: TPMI_DH_OBJECT.plus()
    bind: TPMI_DH_ENTITY.plus()


@tpm_dataclass
class TPMS_COMMAND_HANDLES_VERIFY_SIGNATURE:
    keyHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_ECC_PARAMETERS:
    pass


@tpm_dataclass
class TPMS_COMMAND_HANDLES_FIRMWARE_READ:
    pass


@tpm_dataclass
class TPMS_COMMAND_HANDLES_GET_CAPABILITY:
    pass


@tpm_dataclass
class TPMS_COMMAND_HANDLES_GET_RANDOM:
    pass


@tpm_dataclass
class TPMS_COMMAND_HANDLES_GET_TEST_RESULT:
    pass


@tpm_dataclass
class TPMS_COMMAND_HANDLES_HASH:
    pass


@tpm_dataclass
class TPMS_COMMAND_HANDLES_PCR_READ:
    pass


@tpm_dataclass
class TPMS_COMMAND_HANDLES_POLICY_PCR:
    policySession: TPMI_SH_POLICY


@tpm_dataclass
class TPMS_COMMAND_HANDLES_POLICY_RESTART:
    sessionHandle: TPMI_SH_POLICY


@tpm_dataclass
class TPMS_COMMAND_HANDLES_READ_CLOCK:
    pass


@tpm_dataclass
class TPMS_COMMAND_HANDLES_PCR_EXTEND:
    pcrHandle: TPMI_DH_PCR


@tpm_dataclass
class TPMS_COMMAND_HANDLES_PCR_SET_AUTH_VALUE:
    pcrHandle: TPMI_DH_PCR


@tpm_dataclass
class TPMS_COMMAND_HANDLES_NV_CERTIFY:
    signHandle: TPMI_DH_OBJECT
    authHandle: TPMI_RH_NV_AUTH
    nvIndex: TPMI_RH_NV_INDEX


@tpm_dataclass
class TPMS_COMMAND_HANDLES_EVENT_SEQUENCE_COMPLETE:
    pcrHandle: TPMI_DH_PCR
    sequenceHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_HASH_SEQUENCE_START:
    pass


@tpm_dataclass
class TPMS_COMMAND_HANDLES_POLICY_PHYSICAL_PRESENCE:
    policySession: TPMI_SH_POLICY


@tpm_dataclass
class TPMS_COMMAND_HANDLES_POLICY_DUPLICATION_SELECT:
    policySession: TPMI_SH_POLICY


@tpm_dataclass
class TPMS_COMMAND_HANDLES_POLICY_GET_DIGEST:
    policySession: TPMI_SH_POLICY


@tpm_dataclass
class TPMS_COMMAND_HANDLES_TEST_PARMS:
    pass


@tpm_dataclass
class TPMS_COMMAND_HANDLES_COMMIT:
    signHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_POLICY_PASSWORD:
    policySession: TPMI_SH_POLICY


@tpm_dataclass
class TPMS_COMMAND_HANDLES_Z_GEN_2_PHASE:
    keyA: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_EC_EPHEMERAL:
    pass


@tpm_dataclass
class TPMS_COMMAND_HANDLES_POLICY_NV_WRITTEN:
    policySession: TPMI_SH_POLICY


@tpm_dataclass
class TPMS_COMMAND_HANDLES_POLICY_TEMPLATE:
    policySession: TPMI_SH_POLICY


@tpm_dataclass
class TPMS_COMMAND_HANDLES_CREATE_LOADED:
    parentHandle: TPMI_DH_PARENT.plus()


@tpm_dataclass
class TPMS_COMMAND_HANDLES_POLICY_AUTHORIZE_NV:
    authHandle: TPMI_RH_NV_AUTH
    nvIndex: TPMI_RH_NV_INDEX
    policySession: TPMI_SH_POLICY


@tpm_dataclass
class TPMS_COMMAND_HANDLES_ENCRYPT_DECRYPT2:
    keyHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_AC_GET_CAPABILITY:
    ac: TPMI_RH_AC


@tpm_dataclass
class TPMS_COMMAND_HANDLES_AC_SEND:
    sendObject: TPMI_DH_OBJECT
    authHandle: TPMI_RH_NV_AUTH
    ac: TPMI_RH_AC


@tpm_dataclass
class TPMS_COMMAND_HANDLES_POLICY_AC_SEND_SELECT:
    policySession: TPMI_SH_POLICY


@tpm_dataclass
class TPMS_COMMAND_HANDLES_CERTIFY_X509:
    objectHandle: TPMI_DH_OBJECT
    signHandle: TPMI_DH_OBJECT


@tpm_dataclass
class TPMS_COMMAND_HANDLES_ACT_SET_TIMEOUT:
    actHandle: TPMI_RH_ACT


command_handle_types = {
    TPM_CC.NV_UndefineSpaceSpecial: TPMS_COMMAND_HANDLES_NV_UNDEFINE_SPACE_SPECIAL,
    TPM_CC.EvictControl: TPMS_COMMAND_HANDLES_EVICT_CONTROL,
    TPM_CC.HierarchyControl: TPMS_COMMAND_HANDLES_HIERARCHY_CONTROL,
    TPM_CC.NV_UndefineSpace: TPMS_COMMAND_HANDLES_NV_UNDEFINE_SPACE,
    TPM_CC.ChangeEPS: TPMS_COMMAND_HANDLES_CHANGE_EPS,
    TPM_CC.ChangePPS: TPMS_COMMAND_HANDLES_CHANGE_PPS,
    TPM_CC.Clear: TPMS_COMMAND_HANDLES_CLEAR,
    TPM_CC.ClearControl: TPMS_COMMAND_HANDLES_CLEAR_CONTROL,
    TPM_CC.ClockSet: TPMS_COMMAND_HANDLES_CLOCK_SET,
    TPM_CC.HierarchyChangeAuth: TPMS_COMMAND_HANDLES_HIERARCHY_CHANGE_AUTH,
    TPM_CC.NV_DefineSpace: TPMS_COMMAND_HANDLES_NV_DEFINE_SPACE,
    TPM_CC.PCR_Allocate: TPMS_COMMAND_HANDLES_PCR_ALLOCATE,
    TPM_CC.PCR_SetAuthPolicy: TPMS_COMMAND_HANDLES_PCR_SET_AUTH_POLICY,
    TPM_CC.PP_Commands: TPMS_COMMAND_HANDLES_PP_COMMANDS,
    TPM_CC.SetPrimaryPolicy: TPMS_COMMAND_HANDLES_SET_PRIMARY_POLICY,
    TPM_CC.FieldUpgradeStart: TPMS_COMMAND_HANDLES_FIELD_UPGRADE_START,
    TPM_CC.ClockRateAdjust: TPMS_COMMAND_HANDLES_CLOCK_RATE_ADJUST,
    TPM_CC.CreatePrimary: TPMS_COMMAND_HANDLES_CREATE_PRIMARY,
    TPM_CC.NV_GlobalWriteLock: TPMS_COMMAND_HANDLES_NV_GLOBAL_WRITE_LOCK,
    TPM_CC.GetCommandAuditDigest: TPMS_COMMAND_HANDLES_GET_COMMAND_AUDIT_DIGEST,
    TPM_CC.NV_Increment: TPMS_COMMAND_HANDLES_NV_INCREMENT,
    TPM_CC.NV_SetBits: TPMS_COMMAND_HANDLES_NV_SET_BITS,
    TPM_CC.NV_Extend: TPMS_COMMAND_HANDLES_NV_EXTEND,
    TPM_CC.NV_Write: TPMS_COMMAND_HANDLES_NV_WRITE,
    TPM_CC.NV_WriteLock: TPMS_COMMAND_HANDLES_NV_WRITE_LOCK,
    TPM_CC.DictionaryAttackLockReset: TPMS_COMMAND_HANDLES_DICTIONARY_ATTACK_LOCK_RESET,
    TPM_CC.DictionaryAttackParameters: TPMS_COMMAND_HANDLES_DICTIONARY_ATTACK_PARAMETERS,
    TPM_CC.NV_ChangeAuth: TPMS_COMMAND_HANDLES_NV_CHANGE_AUTH,
    TPM_CC.PCR_Event: TPMS_COMMAND_HANDLES_PCR_EVENT,
    TPM_CC.PCR_Reset: TPMS_COMMAND_HANDLES_PCR_RESET,
    TPM_CC.SequenceComplete: TPMS_COMMAND_HANDLES_SEQUENCE_COMPLETE,
    TPM_CC.SetAlgorithmSet: TPMS_COMMAND_HANDLES_SET_ALGORITHM_SET,
    TPM_CC.SetCommandCodeAuditStatus: TPMS_COMMAND_HANDLES_SET_COMMAND_CODE_AUDIT_STATUS,
    TPM_CC.FieldUpgradeData: TPMS_COMMAND_HANDLES_FIELD_UPGRADE_DATA,
    TPM_CC.IncrementalSelfTest: TPMS_COMMAND_HANDLES_INCREMENTAL_SELF_TEST,
    TPM_CC.SelfTest: TPMS_COMMAND_HANDLES_SELF_TEST,
    TPM_CC.Startup: TPMS_COMMAND_HANDLES_STARTUP,
    TPM_CC.Shutdown: TPMS_COMMAND_HANDLES_SHUTDOWN,
    TPM_CC.StirRandom: TPMS_COMMAND_HANDLES_STIR_RANDOM,
    TPM_CC.ActivateCredential: TPMS_COMMAND_HANDLES_ACTIVATE_CREDENTIAL,
    TPM_CC.Certify: TPMS_COMMAND_HANDLES_CERTIFY,
    TPM_CC.PolicyNV: TPMS_COMMAND_HANDLES_POLICY_NV,
    TPM_CC.CertifyCreation: TPMS_COMMAND_HANDLES_CERTIFY_CREATION,
    TPM_CC.Duplicate: TPMS_COMMAND_HANDLES_DUPLICATE,
    TPM_CC.GetTime: TPMS_COMMAND_HANDLES_GET_TIME,
    TPM_CC.GetSessionAuditDigest: TPMS_COMMAND_HANDLES_GET_SESSION_AUDIT_DIGEST,
    TPM_CC.NV_Read: TPMS_COMMAND_HANDLES_NV_READ,
    TPM_CC.NV_ReadLock: TPMS_COMMAND_HANDLES_NV_READ_LOCK,
    TPM_CC.ObjectChangeAuth: TPMS_COMMAND_HANDLES_OBJECT_CHANGE_AUTH,
    TPM_CC.PolicySecret: TPMS_COMMAND_HANDLES_POLICY_SECRET,
    TPM_CC.Rewrap: TPMS_COMMAND_HANDLES_REWRAP,
    TPM_CC.Create: TPMS_COMMAND_HANDLES_CREATE,
    TPM_CC.ECDH_ZGen: TPMS_COMMAND_HANDLES_ECDH_Z_GEN,
    TPM_CC.HMAC: TPMS_COMMAND_HANDLES_HMAC,
    TPM_CC.Import: TPMS_COMMAND_HANDLES_IMPORT,
    TPM_CC.Load: TPMS_COMMAND_HANDLES_LOAD,
    TPM_CC.Quote: TPMS_COMMAND_HANDLES_QUOTE,
    TPM_CC.RSA_Decrypt: TPMS_COMMAND_HANDLES_RSA_DECRYPT,
    TPM_CC.HMAC_Start: TPMS_COMMAND_HANDLES_HMAC_START,
    TPM_CC.SequenceUpdate: TPMS_COMMAND_HANDLES_SEQUENCE_UPDATE,
    TPM_CC.Sign: TPMS_COMMAND_HANDLES_SIGN,
    TPM_CC.Unseal: TPMS_COMMAND_HANDLES_UNSEAL,
    TPM_CC.PolicySigned: TPMS_COMMAND_HANDLES_POLICY_SIGNED,
    TPM_CC.ContextLoad: TPMS_COMMAND_HANDLES_CONTEXT_LOAD,
    TPM_CC.ContextSave: TPMS_COMMAND_HANDLES_CONTEXT_SAVE,
    TPM_CC.ECDH_KeyGen: TPMS_COMMAND_HANDLES_ECDH_KEY_GEN,
    TPM_CC.EncryptDecrypt: TPMS_COMMAND_HANDLES_ENCRYPT_DECRYPT,
    TPM_CC.FlushContext: TPMS_COMMAND_HANDLES_FLUSH_CONTEXT,
    TPM_CC.LoadExternal: TPMS_COMMAND_HANDLES_LOAD_EXTERNAL,
    TPM_CC.MakeCredential: TPMS_COMMAND_HANDLES_MAKE_CREDENTIAL,
    TPM_CC.NV_ReadPublic: TPMS_COMMAND_HANDLES_NV_READ_PUBLIC,
    TPM_CC.PolicyAuthorize: TPMS_COMMAND_HANDLES_POLICY_AUTHORIZE,
    TPM_CC.PolicyAuthValue: TPMS_COMMAND_HANDLES_POLICY_AUTH_VALUE,
    TPM_CC.PolicyCommandCode: TPMS_COMMAND_HANDLES_POLICY_COMMAND_CODE,
    TPM_CC.PolicyCounterTimer: TPMS_COMMAND_HANDLES_POLICY_COUNTER_TIMER,
    TPM_CC.PolicyCpHash: TPMS_COMMAND_HANDLES_POLICY_CP_HASH,
    TPM_CC.PolicyLocality: TPMS_COMMAND_HANDLES_POLICY_LOCALITY,
    TPM_CC.PolicyNameHash: TPMS_COMMAND_HANDLES_POLICY_NAME_HASH,
    TPM_CC.PolicyOR: TPMS_COMMAND_HANDLES_POLICY_OR,
    TPM_CC.PolicyTicket: TPMS_COMMAND_HANDLES_POLICY_TICKET,
    TPM_CC.ReadPublic: TPMS_COMMAND_HANDLES_READ_PUBLIC,
    TPM_CC.RSA_Encrypt: TPMS_COMMAND_HANDLES_RSA_ENCRYPT,
    TPM_CC.StartAuthSession: TPMS_COMMAND_HANDLES_START_AUTH_SESSION,
    TPM_CC.VerifySignature: TPMS_COMMAND_HANDLES_VERIFY_SIGNATURE,
    TPM_CC.ECC_Parameters: TPMS_COMMAND_HANDLES_ECC_PARAMETERS,
    TPM_CC.FirmwareRead: TPMS_COMMAND_HANDLES_FIRMWARE_READ,
    TPM_CC.GetCapability: TPMS_COMMAND_HANDLES_GET_CAPABILITY,
    TPM_CC.GetRandom: TPMS_COMMAND_HANDLES_GET_RANDOM,
    TPM_CC.GetTestResult: TPMS_COMMAND_HANDLES_GET_TEST_RESULT,
    TPM_CC.Hash: TPMS_COMMAND_HANDLES_HASH,
    TPM_CC.PCR_Read: TPMS_COMMAND_HANDLES_PCR_READ,
    TPM_CC.PolicyPCR: TPMS_COMMAND_HANDLES_POLICY_PCR,
    TPM_CC.PolicyRestart: TPMS_COMMAND_HANDLES_POLICY_RESTART,
    TPM_CC.ReadClock: TPMS_COMMAND_HANDLES_READ_CLOCK,
    TPM_CC.PCR_Extend: TPMS_COMMAND_HANDLES_PCR_EXTEND,
    TPM_CC.PCR_SetAuthValue: TPMS_COMMAND_HANDLES_PCR_SET_AUTH_VALUE,
    TPM_CC.NV_Certify: TPMS_COMMAND_HANDLES_NV_CERTIFY,
    TPM_CC.EventSequenceComplete: TPMS_COMMAND_HANDLES_EVENT_SEQUENCE_COMPLETE,
    TPM_CC.HashSequenceStart: TPMS_COMMAND_HANDLES_HASH_SEQUENCE_START,
    TPM_CC.PolicyPhysicalPresence: TPMS_COMMAND_HANDLES_POLICY_PHYSICAL_PRESENCE,
    TPM_CC.PolicyDuplicationSelect: TPMS_COMMAND_HANDLES_POLICY_DUPLICATION_SELECT,
    TPM_CC.PolicyGetDigest: TPMS_COMMAND_HANDLES_POLICY_GET_DIGEST,
    TPM_CC.TestParms: TPMS_COMMAND_HANDLES_TEST_PARMS,
    TPM_CC.Commit: TPMS_COMMAND_HANDLES_COMMIT,
    TPM_CC.PolicyPassword: TPMS_COMMAND_HANDLES_POLICY_PASSWORD,
    TPM_CC.ZGen_2Phase: TPMS_COMMAND_HANDLES_Z_GEN_2_PHASE,
    TPM_CC.EC_Ephemeral: TPMS_COMMAND_HANDLES_EC_EPHEMERAL,
    TPM_CC.PolicyNvWritten: TPMS_COMMAND_HANDLES_POLICY_NV_WRITTEN,
    TPM_CC.PolicyTemplate: TPMS_COMMAND_HANDLES_POLICY_TEMPLATE,
    TPM_CC.CreateLoaded: TPMS_COMMAND_HANDLES_CREATE_LOADED,
    TPM_CC.PolicyAuthorizeNV: TPMS_COMMAND_HANDLES_POLICY_AUTHORIZE_NV,
    TPM_CC.EncryptDecrypt2: TPMS_COMMAND_HANDLES_ENCRYPT_DECRYPT2,
    TPM_CC.AC_GetCapability: TPMS_COMMAND_HANDLES_AC_GET_CAPABILITY,
    TPM_CC.AC_Send: TPMS_COMMAND_HANDLES_AC_SEND,
    TPM_CC.Policy_AC_SendSelect: TPMS_COMMAND_HANDLES_POLICY_AC_SEND_SELECT,
    TPM_CC.CertifyX509: TPMS_COMMAND_HANDLES_CERTIFY_X509,
    TPM_CC.ACT_SetTimeout: TPMS_COMMAND_HANDLES_ACT_SET_TIMEOUT,
}
